#!/usr/bin/env python

import os
import sys
import logging
from celery_connectors.utils import ev
from network_pipeline.log.setup_logging import setup_logging
from network_pipeline.consts import VALID
from network_pipeline.build_training_request import build_training_request
from keras.models import Sequential
from keras.layers import Dense


setup_logging(config_name="modelers.json")
name = "keras-dnn"
log = logging.getLogger(name)
log.info("start - {}".format(name))


csv_file = ev(
    "CSV_FILE",
    "/tmp/cleaned_attack_scans.csv")
meta_file = ev(
    "CSV_META_FILE",
    "/tmp/cleaned_metadata.json")
predict_feature = ev(
    "PREDICT_FEATURE",
    "label_value")
test_size = float(ev(
    "TEST_SIZE",
    "0.20"))

if not os.path.exists(csv_file):
    log.error(("missing csv_file={}")
              .format(csv_file))
    sys.exit(1)

res = build_training_request(
        csv_file=csv_file,
        meta_file=meta_file,
        predict_feature=predict_feature,
        test_size=test_size)

if res["status"] != VALID:
    log.error(("Stopping for status={} "
               "errors: {}")
              .format(res["status"],
                      res["err"]))
    sys.exit(1)
else:
    log.info(("built_training_request={} "
              "features={} ignore={}")
             .format(res["status"],
                     res["features_to_process"],
                     res["ignore_features"]))
# end of validating the training request

log.info("ready for training")

log.info("creating Keras - sequential model")

# create the model
model = Sequential()
model.add(Dense(8,
                input_dim=len(res["features_to_process"]),
                kernel_initializer="uniform",
                activation="relu"))
model.add(Dense(6,
                kernel_initializer="uniform",
                activation="relu"))
model.add(Dense(1,
                kernel_initializer="uniform",
                activation="sigmoid"))

log.info("compiling model")

# compile the model
model.compile(loss="binary_crossentropy",
              optimizer="adam",
              metrics=["accuracy"])

log.info("fitting model - please wait")

# fit the model
model.fit(res["X_train"],
          res["Y_train"],
          validation_data=(res["X_test"],
                           res["Y_test"]),
          epochs=50,
          batch_size=2,
          verbose=1)

# evaluate the model
scores = model.evaluate(res["X_test"],
                        res["Y_test"])
log.info(("Accuracy: {}")
         .format(scores[1] * 100))



import os
import logging
import json
import numpy as np
import pandas as pd
from celery_connectors.utils import ev
from network_pipeline.log.setup_logging import setup_logging
from network_pipeline.consts import VALID
from network_pipeline.consts import INVALID
from network_pipeline.consts import ERROR
from sklearn.model_selection import train_test_split


setup_logging(config_name="modelers.json")
name = "training-utils"
log = logging.getLogger(name)


def build_training_request(
        csv_file=ev(
            "CSV_FILE",
            "/tmp/cleaned_attack_scans.csv"),
        meta_file=ev(
            "CSV_META_FILE",
            "/tmp/cleaned_metadata.json"),
        predict_feature=ev(
            "PREDICT_FEATURE",
            "label_value"),
        ignore_features=[
            "label_name",
            "ip_src",   # need to make this an int
            "ip_dst",   # need to make this an int
            "eth_src",  # need to make this an int
            "eth_dst"   # need to make this an int
        ],
        seed=None,
        test_size=float(ev(
            "TEST_SIZE",
            "0.20")),
        preproc_rules=None):
    """build_training_request

    :param csv_file: csv file built with prepare-dataset.py
    :param meta_file: metadata file built with prepare-dataset.py
    :param predict_feature: feature (column) to predict
    :param ignore_features: features to remove from the csv
                            before the split of test + train
                            data
    :param seed: integer to seed
    :param test_size: percent of records to split into test
                      vs train
    :param preproc_rules: future preprocessing rules hooks
    """

    last_step = "not started"
    res = {
        "status": INVALID,
        "err": "",
        "csv_file": csv_file,
        "meta_file": meta_file,
        "meta_data": None,
        "seed": None,
        "test_size": test_size,
        "predict_feature": predict_feature,
        "features_to_process": [],
        "ignore_features": ignore_features,
        "X_train": None,
        "X_test": None,
        "Y_train": None,
        "Y_test": None
    }

    try:

        last_step = ("building seed={}").format(
                        seed)

        log.debug(last_step)

        use_seed = seed
        if not use_seed:
            use_seed = 9

        res["seed"] = np.random.seed(use_seed)

        last_step = ("Loading csv={}").format(
                        csv_file)

        log.info(last_step)

        if not os.path.exists(csv_file):
            res["status"] = ERROR
            res["err"] = ("Unable to find csv_file={}").format(
                            csv_file)
            log.error(res["err"])
            return res
        # end of checking for a valid csv file on disk

        if not os.path.exists(meta_file):
            res["status"] = ERROR
            res["err"] = ("Unable to find meta_file={}").format(
                            meta_file)
            log.error(res["err"])
            return res
        # end of checking for a valid metadata file on disk

        # load csv file into pandas dataframe
        df = pd.read_csv(csv_file)

        features_to_process = []
        meta_data = {}

        try:
            last_step = ("opening metadata={}").format(
                            meta_file)
            log.debug(last_step)
            meta_data = json.loads(
                open(meta_file, "r").read()
            )
            res["meta_data"] = meta_data
            if "post_proc_rules" in meta_data:
                if "drop_columns" in meta_data["post_proc_rules"]:
                    log.debug(("Found drop_columns={}")
                              .format(
                                meta_data["post_proc_rules"]["drop_columns"]
                              ))
                    for ign in meta_data["post_proc_rules"]["drop_columns"]:
                        ignore_features.append(ign)
        except Exception as e:
            res["error"] = ("Failed building ignore_features: "
                            "ignore_features={} meta={} meta_data={} "
                            "last_step='{}' ex='{}'").format(
                                ignore_features,
                                meta_file,
                                meta_data,
                                last_step,
                                e)
            log.error(res["error"])
            res["status"] = ERROR
            return res
        # end of trying to lookup the meta data file
        # for non-int/float features to ignore

        last_step = ("metadata={} df has "
                     "columns={} ignore={}").format(
                        meta_file,
                        df.columns.values,
                        ignore_features)

        log.info(last_step)

        for feature in df.columns.values:
            keep_it = True
            for ign in ignore_features:
                if feature == ign:
                    keep_it = False
            if keep_it:
                features_to_process.append(feature)
        # end of for all features to process

        last_step = ("Done post-procecessing "
                     "Predicting={} with features={} "
                     "ignore_features={} records={}").format(
                        predict_feature,
                        features_to_process,
                        ignore_features,
                        len(df.index))

        log.info(last_step)

        res["predict_feature"] = predict_feature
        res["features_to_process"] = features_to_process
        res["ignore_features"] = ignore_features

        # split the data into training
        (res["X_train"],
         res["X_test"],
         res["Y_train"],
         res["Y_test"]) = train_test_split(
                            df[features_to_process],
                            df[predict_feature],
                            test_size=test_size,
                            random_state=res["seed"])

        last_step = ("Done splitting rows={} into "
                     "X_train={} X_test={} "
                     "Y_train={} Y_test={}").format(
                        len(df.index),
                        len(res["X_train"]),
                        len(res["X_test"]),
                        len(res["Y_train"]),
                        len(res["Y_test"]))

        log.info(("Success: {}")
                 .format(last_step))

        res["err"] = ""
        res["status"] = VALID
    except Exception as e:
        res["status"] = ERROR
        res["err"] = ("Failed build_training_request "
                      "step='{}' with ex='{}'").format(
                        last_step,
                        e)
        log.error(("build_training_request: {}")
                  .format(res["err"]))
    # end of try/ex

    return res
# end of build_training_request



#!/usr/bin/env python

import logging
from celery_connectors.kombu_subscriber import KombuSubscriber
from network_pipeline.consts import FORWARD_BROKER_URL
from network_pipeline.consts import FORWARD_SSL_OPTIONS
from network_pipeline.consts import FORWARD_QUEUE
from network_pipeline.log.setup_logging import setup_logging
from network_pipeline.record_packets_to_csv import RecordPacketsToCSV


setup_logging(config_name="packets-rabbitmq-logging.json")
name = "kombu-rmq-sub"
log = logging.getLogger(name)

log.info("start - {}".format(name))

agg = RecordPacketsToCSV()


def recv_msg(body,
             message):
    """recv_msg

    :param body: message body
    :param message: message object can ack, requeue or reject
    """

    log.info(("callback received msg "))

    agg.handle_msg(body=body,
                   org_message=message)
# end of recv_message


# end of recv_message
# Initialize KombuSubscriber
sub = KombuSubscriber(name,
                      FORWARD_BROKER_URL,
                      FORWARD_SSL_OPTIONS)


# Now consume:
seconds_to_consume = 10.0
heartbeat = 60
serializer = "application/json"
queue = FORWARD_QUEUE

sub.consume(callback=recv_msg,
            queue=queue,
            exchange=None,
            routing_key=None,
            serializer=serializer,
            heartbeat=heartbeat,
            time_to_wait=seconds_to_consume)

log.info("end - {}".format(name))



#!/usr/bin/env python

import os
import socket
import uuid
import datetime

host = os.getenv(
            "UDP_SEND_TO_HOST",
            "0.0.0.0").strip().lstrip()
port = int(os.getenv(
            "UDP_SEND_TO_PORT",
            "17000").strip().lstrip())

need_response = os.getenv("NEED_RESPONSE", "0") == "1"

if __name__ == '__main__':
    msg = os.getenv(
            "MSG",
            "testing UDP msg time={} - {}".format(
                datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
                uuid.uuid4()))

    server_address = (host, port)

    client = socket.socket(socket.AF_INET,
                           socket.SOCK_DGRAM)

    print(("sending UDP: "
           "address={} msg={}")
          .format(server_address,
                  msg))
    client.sendto(msg.encode(), server_address)
    if need_response:
        data = client.recv(1024).decode()
    client.close()



#!/usr/bin/env python

import logging
import scapy.all as scapy
from celery_connectors.utils import ev
from network_pipeline.log.setup_logging import setup_logging
from network_pipeline.handle_packets import handle_packets


setup_logging()
# scapy capture agent
name = "cap-udp"
log = logging.getLogger(name)

dev = ev("CAP_DEVICE",
         "lo")

"""
Ignore ports for forwarding to consolidators:

Redis VM: 6379, 16379
RabbitMQ VM: 5672, 15672, 25672

"""

# http://biot.com/capstats/bpf.html
default_filter = "udp"
custom_filter = ev("NETWORK_FILTER",
                   default_filter)

log.info(("starting device={} filter={}")
         .format(dev,
                 custom_filter))

scapy.sniff(
            filter=custom_filter,
            prn=handle_packets)

log.info("done")



import logging
import multiprocessing
from network_pipeline.log.setup_logging import setup_logging

setup_logging()
name = "start-consumers"
log = logging.getLogger(name)


def start_consumers_for_queue(prefix_name="worker",
                              num_workers=2,
                              tasks=None,
                              queue_to_consume=None,
                              shutdown_msg="SHUTDOWN",
                              consumer_class=None,
                              need_response=False,
                              callback=None):

    consumers = []

    if not consumer_class:
        log.error("Please provide a consumer_class arg")
        log.error("  like: network_pipeline.packet_consumer.PacketConsumer")
        return consumers

    if not tasks:
        log.error("Missing tasks")
        return consumers

    if not queue_to_consume:
        log.error("Missing queue")
        return consumers

    # Establish communication queues
    log.info(("Creating consumers={} for cores={}")
             .format(multiprocessing.cpu_count(),
                     num_workers))

    for i in range(num_workers):
        consumers.append(consumer_class(
                            "{}-{}".format(prefix_name,
                                           i + 1),
                            tasks,
                            queue_to_consume,
                            shutdown_msg=shutdown_msg,
                            need_response=need_response,
                            callback=callback))

    log.info("Starting consumers={}".format(len(consumers)))
    for w in consumers:
        w.start()

    return consumers
# end of start_consumers_for_queue



#!/usr/bin/env python

import os
import socket
import uuid
import datetime

need_response = os.getenv("NEED_RESPONSE", "0") == "1"

if __name__ == '__main__':
    msg = os.getenv(
            "MSG",
            "testing msg time={} - {}".format(
                datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
                uuid.uuid4()))
    host = "127.0.0.1"
    port = 80

    client = socket.socket()
    client.connect((host, port))

    print(("sending: {}")
          .format(msg))
    client.send(msg.encode())
    if need_response:
        data = client.recv(1024).decode()
    client.close()



#!/usr/bin/env python

import os
import sys
import datetime
import time
import socket

host = os.getenv(
            "UDP_LISTEN_ON_HOST",
            "127.0.0.1").strip().lstrip()
port = int(os.getenv(
            "UDP_LISTEN_ON_PORT",
            "17000").strip().lstrip())
backlog = int(os.getenv(
            "UDP_LISTEN_BACKLOG",
            "5").strip().lstrip())
size = int(os.getenv(
            "UDP_LISTEN_SIZE",
            "1024").strip().lstrip())
sleep_in_seconds = float(os.getenv(
            "UDP_LISTEN_SLEEP",
            "0.5").strip().lstrip())
shutdown_hook = os.getenv(
            "UDP_LISTEN_SHUTDOWN_HOOK",
            "/tmp/udp-shutdown-listen-server-{}-{}".format(
                host,
                port)).strip().lstrip()

if os.path.exists(shutdown_hook):
    print(("Please remove the UDP shutdown hook file: "
           "\nrm -f {}")
          .format(shutdown_hook))
    sys.exit(1)

now = datetime.datetime.now().isoformat()
print(("{} - Starting UDP Server address={}:{} "
       "backlog={} size={} sleep={} shutdown={}")
      .format(now,
              host,
              port,
              backlog,
              size,
              sleep_in_seconds,
              shutdown_hook))

s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
s.bind((host, port))

msg = 0
while 1:
    data = None
    address = None
    ignore_key = None

    data, address = s.recvfrom(4096)

    if data:
        now = datetime.datetime.now().isoformat()
        print(("{} received UDP data={} ")
              .format(now,
                      data))
        msg += 1
        if msg > 1000000:
            msg = 0

        # if address:
        #     client.sendto("PROCESSED", address)
    else:
        time.sleep(sleep_in_seconds)

    if os.path.exists(shutdown_hook):
        now = datetime.datetime.now().isoformat()
        print(("{} detected shutdown "
               "file={}")
              .format(now,
                      shutdown_hook))

# end of loop

print("Shutting down")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
A pure python ping implementation using raw sockets.

(This is Python 3 port of https://github.com/jedie/python-ping)
(Tested and working with python 2.7, should work with 2.6+)

Note that ICMP messages can only be sent from processes running as root
(in Windows, you must run this script as 'Administrator').

Derived from ping.c distributed in Linux's netkit. That code is
copyright (c) 1989 by The Regents of the University of California.
That code is in turn derived from code written by Mike Muuss of the
US Army Ballistic Research Laboratory in December, 1983 and
placed in the public domain. They have my thanks.

Bugs are naturally mine. I'd be glad to hear about them. There are
certainly word - size dependencies here.

Copyright (c) Matthew Dixon Cowles, <http://www.visi.com/~mdc/>.
Distributable under the terms of the GNU General Public License
version 2. Provided with no warranties of any sort.

Original Version from Matthew Dixon Cowles:
   -> ftp://ftp.visi.com/users/mdc/ping.py

Rewrite by Jens Diemer:
   -> http://www.python-forum.de/post-69122.html#69122

Rewrite by George Notaras:
   -> http://www.g-loaded.eu/2009/10/30/python-ping/

Enhancements by Martin Falatic:
   -> http://www.falatic.com/index.php/39/pinging-with-python

Enhancements and fixes by Georgi Kolev:
   -> http://github.com/jedie/python-ping/

Bug fix by Andrejs Rozitis:
   -> http://github.com/rozitis/python-ping/

Revision history
~~~~~~~~~~~~~~~~
May 1, 2014
-----------
Little modifications by Mohammad Emami <emamirazavi@gmail.com>
- Added Python 3 support. For now this project will just support
   python 3.x
- Tested with python 3.3
- version was upped to 0.6

March 19, 2013
--------------
* Fixing bug to prevent divide by 0 during run-time.

January 26, 2012
----------------
* Fixing BUG #4 - competability with python 2.x [tested with 2.7]
   - Packet data building is different for 2.x and 3.x.
   'cose of the string/bytes difference.
* Fixing BUG #10 - the multiple resolv issue.
   - When pinging domain names insted of hosts (for exmaple google.com)
   you can get different IP every time you try to resolv it, we should
   resolv the host only once and stick to that IP.
* Fixing BUGs #3 #10 - Doing hostname resolv only once.
* Fixing BUG #14 - Removing all 'global' stuff.
   - You should not use globul! Its bad for you...and its not thread safe!
* Fix - forcing the use of different times on linux/windows for
        more accurate mesurments. (time.time - linux/ time.clock - windows)
* Adding quiet_ping function - This way we'll be able to use this script
    as external lib.
* Changing default timeout to 3s. (1second is not enought)
* Switching data syze to packet size. It's easyer for the user to ignore the
    fact that the packet headr is 8b and the datasize 64 will make packet with
    size 72.

October 12, 2011
--------------
Merged updates from the main project
    -> https://github.com/jedie/python-ping

September 12, 2011
--------------
Bugfixes + cleanup by Jens Diemer
Tested with Ubuntu + Windows 7

September 6, 2011
--------------
Cleanup by Martin Falatic. Restored lost comments and docs. Improved
functionality: constant time between pings, internal times consistently
use milliseconds. Clarified annotations (e.g., in the checksum routine).
Using unsigned data in IP & ICMP header pack/unpack unless otherwise
necessary. Signal handling. Ping-style output formatting and stats.

August 3, 2011
--------------
Ported to py3k by Zach Ware. Mostly done by 2to3; also minor changes to
deal with bytes vs. string changes (no more ord() in checksum() because
>source_string< is actually bytes, added .encode() to data in
send_one_ping()).  That's about it.

March 11, 2010
--------------
changes by Samuel Stauffer:
- replaced time.clock with default_timer which is set to
    time.clock on windows and time.time on other systems.

November 8, 2009
----------------
Improved compatibility with GNU/Linux systems.

Fixes by:
    * George Notaras -- http://www.g-loaded.eu
Reported by:
    * Chris Hallman -- http://cdhallman.blogspot.com

Changes in this release:
    - Re-use time.time() instead of time.clock(). The 2007 implementation
    worked only under Microsoft Windows. Failed on GNU/Linux.
    time.clock() behaves differently under the two OSes[1].

[1] http://docs.python.org/library/time.html#time.clock

May 30, 2007
------------
little rewrite by Jens Diemer:
    -  change socket asterisk import to a normal import
    -  replace time.time() with time.clock()
    -  delete "return None" (or change to "return" only)
    -  in checksum() rename "str" to "source_string"

December 4, 2000
----------------
Changed the struct.pack() calls to pack the checksum and ID as
unsigned. My thanks to Jerome Poincheval for the fix.

November 22, 1997
-----------------
Initial hack. Doesn't do much, but rather than try to guess
what features I (or others) will want in the future, I've only
put in what I need now.

December 16, 1997
-----------------
For some reason, the checksum bytes are in the wrong order when
this is run under Solaris 2.X for SPARC but it works right under
Linux x86. Since I don't know just what's wrong, I'll swap the
bytes always and then do an htons().

===========================================================================
IP header info from RFC791
    -> http://tools.ietf.org/html/rfc791)

0                   1                   2                   3
0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|Version|  IHL  |Type of Service|          Total Length         |
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|         Identification        |Flags|      Fragment Offset    |
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|  Time to Live |    Protocol   |         Header Checksum       |
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|                       Source Address                          |
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|                    Destination Address                        |
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|                    Options                    |    Padding    |
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+

===========================================================================
ICMP Echo / Echo Reply Message header info from RFC792
    -> http://tools.ietf.org/html/rfc792

    0                   1                   2                   3
    0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    |     Type      |     Code      |          Checksum             |
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    |           Identifier          |        Sequence Number        |
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    |     Data ...
    +-+-+-+-+-

===========================================================================
ICMP parameter info:
    -> http://www.iana.org/assignments/icmp-parameters/icmp-parameters.xml

===========================================================================
An example of ping's typical output:

PING heise.de (193.99.144.80): 56 data bytes
64 bytes from 193.99.144.80: icmp_seq=0 ttl=240 time=127 ms
64 bytes from 193.99.144.80: icmp_seq=1 ttl=240 time=127 ms
64 bytes from 193.99.144.80: icmp_seq=2 ttl=240 time=126 ms
64 bytes from 193.99.144.80: icmp_seq=3 ttl=240 time=126 ms
64 bytes from 193.99.144.80: icmp_seq=4 ttl=240 time=127 ms

----heise.de PING Statistics----
5 packets transmitted, 5 packets received, 0.0% packet loss
round-trip (ms)  min/avg/max/med = 126/127/127/127

===========================================================================
"""

import os
import sys
import socket
import struct
import select
import time
import signal

__description__ = 'A pure python ICMP ping implementation using raw sockets.'

if sys.platform == "win32":
    # On Windows, the best timer is time.clock()
    default_timer = time.clock
else:
    # On most other platforms the best timer is time.time()
    default_timer = time.time

NUM_PACKETS = 3
PACKET_SIZE = 64
WAIT_TIMEOUT = 3.0

# ICMP parameters

ICMP_ECHOREPLY = 0  # Echo reply (per RFC792)
ICMP_ECHO = 8  # Echo request (per RFC792)
ICMP_MAX_RECV = 2048  # Max size of incoming buffer

MAX_SLEEP = 1000


class MyStats:
    thisIP = "0.0.0.0"
    pktsSent = 0
    pktsRcvd = 0
    minTime = 999999999
    maxTime = 0
    totTime = 0
    avrgTime = 0
    fracLoss = 1.0


myStats = MyStats  # NOT Used globally anymore.


def checksum(source_string):
    """
    A port of the functionality of in_cksum() from ping.c
    Ideally this would act on the string as a series of 16-bit ints (host
    packed), but this works.
    Network data is big-endian, hosts are typically little-endian
    """
    countTo = (int(len(source_string) / 2)) * 2
    sum = 0
    count = 0

    # Handle bytes in pairs (decoding as short ints)
    loByte = 0
    hiByte = 0
    while count < countTo:
        if (sys.byteorder == "little"):
            loByte = source_string[count]
            hiByte = source_string[count + 1]
        else:
            loByte = source_string[count + 1]
            hiByte = source_string[count]
        try:     # For Python3
            sum = sum + (hiByte * 256 + loByte)
        except Exception:  # For Python2
            sum = sum + (ord(hiByte) * 256 + ord(loByte))
        count += 2

    # Handle last byte if applicable (odd-number of bytes)
    # Endianness should be irrelevant in this case
    if countTo < len(source_string):  # Check for odd length
        loByte = source_string[len(source_string) - 1]
        try:      # For Python3
            sum += loByte
        except Exception:   # For Python2
            sum += ord(loByte)

    # Truncate sum to 32 bits (a variance from ping.c, which
    # uses signed ints, but overflow is unlikely in ping)
    sum &= 0xffffffff

    sum = (sum >> 16) + (sum & 0xffff)    # Add high 16 bits to low 16 bits
    sum += (sum >> 16)                    # Add carry from above (if any)
    answer = ~sum & 0xffff                # Invert and truncate to 16 bits
    answer = socket.htons(answer)

    return answer


def do_one(myStats,
           destIP,
           hostname,
           timeout,
           mySeqNumber,
           packet_size,
           quiet=False):
    """do_one
    Returns either the delay (in ms) or None on timeout.

    :param myStats:
    :param destIP:
    :param hostname:
    :param timeout:
    :param mySeqNumber:
    :param packet_size:
    :param quiet:
    """
    delay = None

    try:
        # One could use UDP here, but it's obscure
        mySocket = socket.socket(socket.AF_INET,
                                 socket.SOCK_RAW,
                                 socket.getprotobyname("icmp"))
    except socket.error as e:
        print("failed. (socket error: '%s')" % e.args[1])
        raise  # raise the original error

    my_ID = os.getpid() & 0xFFFF

    sentTime = send_one_ping(mySocket, destIP, my_ID, mySeqNumber, packet_size)
    if not sentTime:
        mySocket.close()
        return delay

    myStats.pktsSent += 1

    recvTime, dataSize, iphSrcIP, icmpSeqNumber, iphTTL = \
        receive_one_ping(
            mySocket,
            my_ID,
            timeout)

    mySocket.close()

    if recvTime:
        delay = (recvTime - sentTime) * 1000
        if not quiet:
            print("%d bytes from %s: icmp_seq=%d ttl=%d time=%d ms" % (
                dataSize,
                socket.inet_ntoa(struct.pack("!I",
                                             iphSrcIP)),
                icmpSeqNumber,
                iphTTL,
                delay)
            )
        myStats.pktsRcvd += 1
        myStats.totTime += delay
        if myStats.minTime > delay:
            myStats.minTime = delay
        if myStats.maxTime < delay:
            myStats.maxTime = delay
    else:
        delay = None
        print("Request timed out.")

    return delay


def send_one_ping(mySocket, destIP, myID, mySeqNumber, packet_size):
    """
    Send one ping to the given >destIP<.
    """
    # destIP  =  socket.gethostbyname(destIP)

    # Header is type (8), code (8), checksum (16), id (16), sequence (16)
    # (packet_size - 8) - Remove header size from packet size
    myChecksum = 0

    # Make a dummy heder with a 0 checksum.
    header = struct.pack(
        "!BBHHH", ICMP_ECHO, 0, myChecksum, myID, mySeqNumber
    )

    padBytes = []
    startVal = 0x42
    # 'cose of the string/byte changes in python 2/3 we have
    # to build the data differnely for different version
    # or it will make packets with unexpected size.
    if sys.version[:1] == '2':
        bytes = struct.calcsize("d")
        data = ((packet_size - 8) - bytes) * "Q"
        data = struct.pack("d", default_timer()) + data
    else:
        for i in range(startVal, startVal + (packet_size - 8)):
            padBytes += [(i & 0xff)]  # Keep chars in the 0-255 range
        # data = bytes(padBytes)
        data = bytearray(padBytes)

    # Calculate the checksum on the data and the dummy header.
    myChecksum = checksum(header + data)  # Checksum is in network order

    # Now that we have the right checksum, we put that in. It's just easier
    # to make up a new header than to stuff it into the dummy.
    header = struct.pack(
        "!BBHHH", ICMP_ECHO, 0, myChecksum, myID, mySeqNumber
    )

    packet = header + data

    sendTime = default_timer()

    try:
        # Port number is irrelevant for ICMP
        mySocket.sendto(packet, (destIP, 1))
    except socket.error as e:
        print("General failure (%s)" % (e.args[1]))
        return

    return sendTime


def receive_one_ping(mySocket, myID, timeout):
    """receive_one_ping
    Receive the ping from the socket. Timeout = in ms

    :param mySocket:
    :param myID:
    :param timeout:
    """
    timeLeft = timeout / 1000

    while True:  # Loop while waiting for packet or timeout
        startedSelect = default_timer()
        whatReady = select.select([mySocket], [], [], timeLeft)
        howLongInSelect = (default_timer() - startedSelect)
        if whatReady[0] == []:  # Timeout
            return None, 0, 0, 0, 0

        timeReceived = default_timer()

        recPacket, addr = mySocket.recvfrom(ICMP_MAX_RECV)

        ipHeader = recPacket[:20]
        iphVersion, iphTypeOfSvc, iphLength, \
            iphID, iphFlags, iphTTL, iphProtocol, \
            iphChecksum, iphSrcIP, iphDestIP = struct.unpack(
                "!BBHHHBBHII", ipHeader
            )

        icmpHeader = recPacket[20:28]
        icmpType, icmpCode, icmpChecksum, \
            icmpPacketID, icmpSeqNumber = struct.unpack(
                "!BBHHH", icmpHeader
            )

        if icmpPacketID == myID:  # Our packet
            dataSize = len(recPacket) - 28
            # print (len(recPacket.encode()))
            return (timeReceived,
                    (dataSize + 8),
                    iphSrcIP,
                    icmpSeqNumber,
                    iphTTL)

        timeLeft = timeLeft - howLongInSelect
        if timeLeft <= 0:
            return None, 0, 0, 0, 0


def dump_stats(myStats):
    """
    Show stats when pings are done
    """
    print("\n----%s PYTHON PING Statistics----" % (myStats.thisIP))

    if myStats.pktsSent > 0:
        myStats.fracLoss = (myStats.pktsSent - myStats.pktsRcvd) \
                / myStats.pktsSent

    print(("%d packets transmitted, %d packets received, "
           "%0.1f%% packet loss") % (
        myStats.pktsSent,
        myStats.pktsRcvd,
        100.0 * myStats.fracLoss
    ))

    if myStats.pktsRcvd > 0:
        print("round-trip (ms)  min/avg/max = %d/%0.1f/%d" % (
            myStats.minTime,
            myStats.totTime / myStats.pktsRcvd,
            myStats.maxTime
        ))

    print("")
    return


def signal_handler(signum, frame):
    """
    Handle exit via signals
    """
    dump_stats()
    print("\n(Terminated with signal %d)\n" % (signum))
    sys.exit(0)


def verbose_ping(hostname, timeout=WAIT_TIMEOUT, count=NUM_PACKETS,
                 packet_size=PACKET_SIZE, path_finder=False):
    """
    Send >count< ping to >destIP< with the given >timeout< and display
    the result.
    """
    signal.signal(signal.SIGINT, signal_handler)   # Handle Ctrl-C
    if hasattr(signal, "SIGBREAK"):
        # Handle Ctrl-Break e.g. under Windows
        signal.signal(signal.SIGBREAK, signal_handler)

    myStats = MyStats()  # Reset the stats

    mySeqNumber = 0  # Starting value

    try:
        destIP = socket.gethostbyname(hostname)
        print("\nPYTHON PING %s (%s): %d data bytes" %
              (hostname, destIP, packet_size))
    except socket.gaierror as e:
        print("\nPYTHON PING: Unknown host: %s (%s)" %
              (hostname, e.args[1]))
        print()
        return

    myStats.thisIP = destIP

    for i in range(count):
        delay = do_one(myStats,
                       destIP,
                       hostname,
                       timeout,
                       mySeqNumber,
                       packet_size)

        if not delay:
            delay = 0

        mySeqNumber += 1

        # Pause for the remainder of the MAX_SLEEP period (if applicable)
        if (MAX_SLEEP > delay):
            time.sleep((MAX_SLEEP - delay) / 1000)

    dump_stats(myStats)


def quiet_ping(hostname, timeout=WAIT_TIMEOUT, count=NUM_PACKETS,
               packet_size=PACKET_SIZE, path_finder=False):
    """
    Same as verbose_ping, but the results are returned as tuple
    """
    myStats = MyStats()  # Reset the stats
    mySeqNumber = 0  # Starting value

    try:
        destIP = socket.gethostbyname(hostname)
    except socket.gaierror as e:
        return False

    myStats.thisIP = destIP

    # This will send packet that we dont care
    # about 0.5 seconds before it starts
    # acrutally pinging. This is needed in big
    # MAN/LAN networks where you sometimes
    # loose the first packet.
    # (while the switches find the way... :/ )
    if path_finder:
        fakeStats = MyStats()
        do_one(fakeStats, destIP, hostname, timeout,
               mySeqNumber, packet_size, quiet=True)
        time.sleep(0.5)

    for i in range(count):
        delay = do_one(myStats, destIP, hostname, timeout,
                       mySeqNumber, packet_size, quiet=True)

        if not delay:
            delay = 0

        mySeqNumber += 1

        # Pause for the remainder of the MAX_SLEEP period (if applicable)
        if (MAX_SLEEP > delay):
            time.sleep((MAX_SLEEP - delay) / 1000)

    if myStats.pktsSent > 0:
        myStats.fracLoss = (myStats.pktsSent - myStats.pktsRcvd) \
                            / myStats.pktsSent
    if myStats.pktsRcvd > 0:
        myStats.avrgTime = myStats.totTime / myStats.pktsRcvd

    # return tuple(max_rtt, min_rtt, avrg_rtt, percent_lost)
    return myStats.maxTime, myStats.minTime, myStats.avrgTime, myStats.fracLoss


def main():

    dst = os.getenv(
                "ICMP_DST",
                "{}".format("localhost")
            ).strip().lstrip()
    packet_count = int(os.getenv(
                "ICMP_COUNT",
                "{}".format(NUM_PACKETS)
            ).strip().lstrip())
    packet_size = int(os.getenv(
                "ICMP_PACKET_SIZE",
                "{}".format(PACKET_SIZE)
            ).strip().lstrip())
    timeout = float(os.getenv(
                "ICMP_TIMEOUT",
                "2.0"
            ).strip().lstrip())
    silent = bool(os.getenv(
                "ICMP_SILENT",
                "0"
            ).strip().lstrip() == "1")

    ping = verbose_ping
    if silent:
        ping = quiet_ping

    ping(dst,
         timeout=timeout * 1000,
         count=packet_count,
         packet_size=packet_size)


if __name__ == '__main__':
    main()
